@@ -1,17 +1,36 @@
 import os
+import re
+import sys
 import imp
-import fnmatch
+import pwiz
+import peewee
+import pkgutil
 import datetime
+import importlib
 import playhouse
 
 from peewee import *
 from peewee import Node
 from playhouse.migrate import *
 
+from types import ModuleType
+from cStringIO import StringIO
 from termcolor import colored, cprint
 
 
-__version__ = '0.1.1'
+__version__ = '0.1.2'
+
+
+GENERATE_TEMPLATE = '''
+from peewee import (BaseModel, {imports})
+
+def up(migrator):
+    {create}
+    migrator({up})
+
+def down(migrator):
+    migrator({down})
+'''
 
 
 class Migration(Model):
@@ -35,10 +54,11 @@ class CustomMigrator(SchemaMigrator):
     ''' A custom migrator that keeps track of all
     migrations in order to fake them if necessary
     '''
-    def __init__(self, database, path, **kwargs):
+    def __init__(self, database, module, **kwargs):
         SchemaMigrator.__init__(self, database)
         # Set options
-        self.path = path
+        self.module = module
+        self.module_name = self.module.__name__
         self.direction = kwargs.get('direction', 'up')
         self.migration = kwargs.get('migration', None)
         self.fake = kwargs.get('fake', False)
@@ -77,14 +97,15 @@ class CustomMigrator(SchemaMigrator):
             self.apply_migration(self.migration)
         else:
             # Fetch migrations
-            migrations = get_migrations(self.path)
+            path = os.path.dirname(self.module.__file__)
+            migrations = get_migrations(path)
             for migration in migrations:
                 if not self.force and migration.split("_")[0] <= self.last_id:
                     continue
                 self.apply_migration(migration)
                 self.migrations_run += 1
 
-        if self.migrations_run or self.force                  :
+        if self.migrations_run or self.force:
             cprint("\nNumber of migrations run %d" % self.migrations_run, "magenta")
         else:
             cprint("\nDatabase already upto date!", "magenta")
@@ -119,10 +140,9 @@ class CustomMigrator(SchemaMigrator):
                 cprint("Force running this migration again", "yellow")
 
         # Load the module
-        # module_name = "%s.%s" % (migration_path, migration)
+        module_name = "%s.%s" % (self.module_name, migration)
         try:
-            # module = importlib.import_module(module_name)
-            module = imp.load_source(migration, os.path.join(self.path, '%s.py' % migration))
+            module = importlib.import_module(module_name)
             if not hasattr(module, self.direction):
                 raise MigrationException("%s doesn't have %s migration defined" %
                     (migration, self.direction)
@@ -172,6 +192,49 @@ DATABASE_MAP = dict((value, key)
                     for value in DATABASE_ALIASES[key])
 
 
+class Capturing(list):
+    def __enter__(self):
+        self._stdout = sys.stdout
+        sys.stdout = self._stringio = StringIO()
+        return self
+
+    def __exit__(self, *args):
+        self.extend(self._stringio.getvalue().splitlines())
+        sys.stdout = self._stdout
+
+
+class Generator(object):
+    def __init__(self, engine, database, models, **kwargs):
+        # Tables from the DB, generated by pwiz
+        self.db_tables = self.get_pwiz_tables(engine, database)
+        self.model_tables = self.get_tables(models)
+
+    def get_tables(self, models):
+        return { obj._meta.db_table : obj for obj in
+                models.__dict__.itervalues() if
+                isinstance(obj, peewee.BaseModel) and
+                len(obj._meta.fields) > 1
+            }
+
+    def get_pwiz_tables(self, engine, database):
+        introspector = pwiz.make_introspector(engine, database.database,
+            **database.connect_kwargs
+            )
+        with Capturing() as code:
+            introspector.print_models(None)
+        db_model = imp.new_module('db_model')
+        exec('\n'.join(code), db_model.__dict__)
+
+        return self.get_tables(db_model)
+
+    def run(self):
+        for db_table, obj in self.model_tables.iteritems():
+            if not db_table in self.db_tables:
+                # This is a new model that we need to create the table for
+                continue
+            print(obj._meta.fields)
+
+
 def fake_print(self):
     '''
     This is the overridden __str__ method for Operation
@@ -195,33 +258,43 @@ def fake_print(self):
 
     return str(_fake_run())
 
+
 # Monkey Patch the Operation to show SQL
 setattr(Operation, "__str__", fake_print)
 
 
-def find_files(directory, pattern):
-    ''' Find all files in a directory which matches the pattern '''
-    for root, dirs, files in os.walk(directory):
-        for basename in files:
-            if fnmatch.fnmatch(basename, pattern):
-                yield basename
-
-
-def get_migrations(directory):
+def get_migrations(path):
     '''
     In the specified directory, get all the files which match the pattern
-    000_migration.py
+    0001_migration.py
     '''
-    files = []
-    for fname in find_files(directory, '[0-9]*_*.py'):
-        files.append(fname[:-3])
-    return sorted(files, key=lambda name: int(name[2:].split("_")[0]))
+    pattern = re.compile(r"\d+_[\w\d]+")
+    modules = [name for _, name, _ in pkgutil.iter_modules([path])
+                if pattern.match(name)
+            ]
+
+    return sorted(modules, key=lambda name: int(name.split("_")[0]))
 
 
-def migrate(engine, database, path, **kwargs):
+def validate_args(engine, database, module):
+    if engine not in DATABASE_MAP:
+        raise MigrationException('Unrecognized database engine, must be one of: %s' %
+            ', '.join(DATABASE_MAP.keys()))
+
+    if not isinstance(database, peewee.Database):
+        raise MigrationException("Parameter database has to be a peewee database object")
+
+    if not isinstance(module, ModuleType):
+        raise MigrationException("Parameter module has to be a python module")
+
+
+# Public API
+def migrate(engine, database, module, **kwargs):
     '''
     Execute the migrations. Pass in kwargs
     '''
+    validate_args(engine, database, module)
+
     options = {
         'direction': kwargs.get('direction', 'up'),
         'fake': kwargs.get('fake', False),
@@ -230,16 +303,15 @@ def migrate(engine, database, path, **kwargs):
         'transaction': kwargs.get('transaction', True),
     }
 
-    if engine not in DATABASE_MAP:
-        raise MigrationException('Unrecognized database, must be one of: %s' %
-            ', '.join(DATABASE_MAP.keys()))
-
-    if not database:
-        raise MigrationException("Pass in a valid database")
-
-    if not path or not os.path.isdir(path):
-        raise MigrationException("Path to migrations invalid or not readable")
-
     Migration._meta.database = database
-    migrator = DATABASE_MAP[engine](database, path, **options)
+    migrator = DATABASE_MAP[engine](database, module, **options)
     migrator.run()
+
+
+def generate(engine, database, models, **kwargs):
+    '''
+    Generate the migrations by introspecting the db
+    '''
+    validate_args(engine, database, models)
+    generator = Generator(engine, database, models)
+    generator.run()
